﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using SlimDX;

using DX10 = SlimDX.Direct3D10;
using DXGI = SlimDX.DXGI;

namespace TutorialSeries.DirectX10.Chapter2
{
    public partial class MainWindow : Form
    {
        private DX10.Device m_device;
        private DX10.RenderTargetView m_renderTarget;
        private DXGI.SwapChain m_swapChain;
        private DXGI.SwapChainDescription m_swapChainDesc;
        private DXGI.Factory m_factory;
        private bool m_initialized;

        private SimpleTriangle m_simpleTriangle;

        public MainWindow()
        {
            InitializeComponent();

            this.SetStyle(ControlStyles.ResizeRedraw, true);
            this.SetStyle(ControlStyles.AllPaintingInWmPaint, true);
            this.SetStyle(ControlStyles.Opaque, true);
        }

        /// <summary>
        /// Initializes device and other resources needed for rendering. Returns true, if successful.
        /// </summary>
        private bool Initialize3D()
        {
            try
            {
                m_device = new DX10.Device(DX10.DeviceCreationFlags.SingleThreaded);

                m_factory = new DXGI.Factory();

                m_swapChainDesc = new DXGI.SwapChainDescription();
                m_swapChainDesc.OutputHandle = this.Handle;
                m_swapChainDesc.IsWindowed = true;
                m_swapChainDesc.BufferCount = 1;
                m_swapChainDesc.Flags = DXGI.SwapChainFlags.AllowModeSwitch;
                m_swapChainDesc.ModeDescription = new DXGI.ModeDescription(
                    this.Width,
                    this.Height,
                    new Rational(60, 1),
                    DXGI.Format.R8G8B8A8_UNorm);
                m_swapChainDesc.SampleDescription = new DXGI.SampleDescription(1, 0);
                m_swapChainDesc.SwapEffect = DXGI.SwapEffect.Discard;
                m_swapChainDesc.Usage = DXGI.Usage.RenderTargetOutput;

                m_swapChain = new DXGI.SwapChain(m_factory, m_device, m_swapChainDesc);

                DX10.Viewport viewPort = new DX10.Viewport();
                viewPort.X = 0;
                viewPort.Y = 0;
                viewPort.Width = this.Width;
                viewPort.Height = this.Height;
                viewPort.MinZ = 0f;
                viewPort.MaxZ = 1f;

                DX10.Texture2D backBuffer =
                    DX10.Texture2D.FromSwapChain<DX10.Texture2D>(m_swapChain, 0);
                m_renderTarget = new DX10.RenderTargetView(m_device, backBuffer);

                m_device.Rasterizer.SetViewports(viewPort);
                m_device.OutputMerger.SetTargets(m_renderTarget);

                m_simpleTriangle = new SimpleTriangle();
                m_simpleTriangle.LoadResources(m_device);

                m_initialized = true;
            }
            catch (Exception ex)
            {
                MessageBox.Show("Error while initializing Direct3D10: \n" + ex.Message);
                m_initialized = false;
            }

            return m_initialized;
        }

        /// <summary>
        /// Rendering is done during the standard OnPaint event
        /// </summary>
        protected override void OnPaint(PaintEventArgs e)
        {
            base.OnPaint(e);

            if (m_initialized)
            {
                m_device.ClearRenderTargetView(m_renderTarget, new Color4(Color.CornflowerBlue));

                m_simpleTriangle.Render(m_device);

                m_swapChain.Present(0, DXGI.PresentFlags.None);
            }
        }

        /// <summary>
        /// Initialize 3D-Graphics within OnLoad event
        /// </summary>
        protected override void OnLoad(EventArgs e)
        {
            base.OnLoad(e);
            Initialize3D();
        }
    }
}
